DEPRECATED_STATIC_TRIGGER_NAMES = [
    "Trigger-branch-create-events",  # used in 1.1.x
    "Trigger-branch-remove-events",  # used in 1.1.x
    "Trigger-webhook-update-configuration",  # used in 1.1.x
    "Trigger-repository-commit-update-event",  # used in 1.1.x
    "Trigger-schema-update-event",  # used in 1.1.x
]

NAME_SEPARATOR = "::"
